;;;
;;; sxml.sxpath - SXML Query Language
;;;
;;;   This file is mechanically translated for Gauche from
;;;   Oleg Kiselyov's SXPATH SXML query language, with
;;;   Kirill Lisovsky and Dmitry Lizorkin's improvements.
;;;   The original files are:
;;;      sxpathlib.scm,v 3.915
;;;      sxpath.scm,v 1.1
;;;      sxpath-ext.scm,v 1.911
;;;   Public domain.
;;;
;;; $Id: sxpath.scm.in,v 1.3 2003/07/23 12:45:49 shirok Exp $
;;;

(define-module sxml.sxpath
  (use srfi-1)
  (use srfi-2)
  (use srfi-11)
  (use srfi-13)
  (use text.parse)
  (use sxml.adaptor)
  (export sxpath nodeset? as-nodeset sxml:element? ntype-names??
          ntype?? ntype-namespace-id?? sxml:invert node-eq? node-equal?
          node-pos sxml:filter take-until take-after map-union
          node-reverse node-trace select-kids node-self node-join
          node-reduce node-or node-closure
          if-sxpath if-car-sxpath car-sxpath
          sxml:string sxml:boolean sxml:number sxml:string-value
          sxml:node? sxml:attr-list sxml:id sxml:equality-cmp
          sxml:equal? sxml:relational-cmp
          sxml:attribute sxml:child sxml:parent sxml:ancestor
          sxml:ancestor-or-self sxml:descendant sxml:descendant-or-self
          sxml:following sxml:following-siblink sxml:namespace
          sxml:preceding sxml:preceding-sibling
          sxml:child-nodes sxml:child-elements
          ))
(select-module sxml.sxpath)

;;; Generated from Oleg Kiselyov's "sxpathlib.scm"
(define (nodeset? x) (or (and (pair? x) (not (symbol? (car x)))) (null? x)))
(define (as-nodeset x) (if (nodeset? x) x (list x)))
(define (sxml:element? obj) (and (pair? obj) (symbol? (car obj)) (not (memq (car obj) '(|@| |@@| *PI* *COMMENT* *ENTITY*)))))
(define (ntype-names?? crit) (lambda (node) (and (pair? node) (memq (car node) crit))))
(define (ntype?? crit) (case crit ((*) sxml:element?) ((*any*) (lambda (node) #t)) ((*text*) (lambda (node) (string? node))) ((*data*) (lambda (node) (not (pair? node)))) (else (lambda (node) (and (pair? node) (eq? crit (car node)))))))
(define (ntype-namespace-id?? ns-id) (lambda (node) (and (pair? node) (not (memq (car node) '(|@| |@@| *PI* *COMMENT* *ENTITY*))) (let ((nm (symbol->string (car node)))) (cond ((string-rindex nm #\:) => (lambda (pos) (and (= pos (string-length ns-id)) (string-prefix? ns-id nm)))) (else (not ns-id)))))))
(define (sxml:invert pred) (lambda (node) (case (pred node) ((#f '()) node) (else #f))))
(define (node-eq? other) (lambda (node) (eq? other node)))
(define (node-equal? other) (lambda (node) (equal? other node)))
(define (node-pos n) (lambda (nodeset) (cond ((not (nodeset? nodeset)) '()) ((null? nodeset) nodeset) ((eqv? n 1) (list (car nodeset))) ((negative? n) ((node-pos (+ n 1 (length nodeset))) nodeset)) (else (assert (positive? n)) ((node-pos (|--| n)) (cdr nodeset))))))
(define (sxml:filter pred?) (lambda (lst) (let loop ((lst (as-nodeset lst)) (res '())) (if (null? lst) (reverse res) (let ((pred-result (pred? (car lst)))) (loop (cdr lst) (if (and pred-result (not (null? pred-result))) (cons (car lst) res) res)))))))
(define (take-until pred?) (lambda (lst) (let loop ((lst (as-nodeset lst))) (if (null? lst) lst (let ((pred-result (pred? (car lst)))) (if (and pred-result (not (null? pred-result))) '() (cons (car lst) (loop (cdr lst)))))))))
(define (take-after pred?) (lambda (lst) (let loop ((lst (as-nodeset lst))) (if (null? lst) lst (let ((pred-result (pred? (car lst)))) (if (and pred-result (not (null? pred-result))) (cdr lst) (loop (cdr lst))))))))
(define (map-union proc lst) (if (null? lst) lst (let ((proc-res (proc (car lst)))) ((if (nodeset? proc-res) append cons) proc-res (map-union proc (cdr lst))))))
(define node-reverse (lambda (node-or-nodeset) (if (not (nodeset? node-or-nodeset)) (list node-or-nodeset) (reverse node-or-nodeset))))
(define (node-trace title) (lambda (node-or-nodeset) (cout nl "-->" title " :") (pp node-or-nodeset) node-or-nodeset))
(define (select-kids test-pred?) (lambda (node) (cond ((null? node) node) ((not (pair? node)) '()) ((symbol? (car node)) ((sxml:filter test-pred?) (cdr node))) (else (map-union (select-kids test-pred?) node)))))
(define node-self sxml:filter)
(define (node-join . selectors) (lambda (nodeset) (let loop ((nodeset nodeset) (selectors selectors)) (if (null? selectors) nodeset (loop (if (nodeset? nodeset) (map-union (car selectors) nodeset) ((car selectors) nodeset)) (cdr selectors))))))
(define (node-reduce . converters) (lambda (nodeset) (let loop ((nodeset nodeset) (converters converters)) (if (null? converters) nodeset (loop ((car converters) nodeset) (cdr converters))))))
(define (node-or . converters) (lambda (node-or-nodeset) (let loop ((result '()) (converters converters)) (if (null? converters) result (loop (append result (or ((car converters) node-or-nodeset) '())) (cdr converters))))))
(define (node-closure test-pred?) (lambda (node) (let loop ((parent node) (result '())) (if (null? parent) result (loop (sxml:child-elements parent) (append result ((select-kids test-pred?) parent)))))))
(define (sxml:node? node) (not (and (pair? node) (memq (car node) '(|@| |@@|)))))
(define (sxml:attr-list obj) (if (and (sxml:element? obj) (not (null? (cdr obj))) (pair? (cadr obj)) (eq? '|@| (caadr obj))) (cdadr obj) '()))
(define (sxml:attribute test-pred?) (let ((fltr (sxml:filter test-pred?))) (lambda (node) (fltr (apply append (map sxml:attr-list (as-nodeset node)))))))
(define (sxml:child test-pred?) (lambda (node) (cond ((null? node) node) ((not (pair? node)) '()) ((memq (car node) '(*PI* *COMMENT* *ENTITY*)) '()) ((symbol? (car node)) ((sxml:filter test-pred?) (cdr node))) (else (map-union (sxml:child test-pred?) node)))))
(define (sxml:parent test-pred?) (lambda (root-node) (lambda (node) (if (nodeset? node) (map-union ((sxml:parent test-pred?) root-node) node) (let rpt ((pairs (apply append (map (lambda (root-n) (map (lambda (arg) (cons arg root-n)) (append (sxml:attr-list root-n) (sxml:child-nodes root-n)))) (as-nodeset root-node))))) (if (null? pairs) '() (let ((pair (car pairs))) (if (eq? (car pair) node) ((sxml:filter test-pred?) (list (cdr pair))) (rpt (append (map (lambda (arg) (cons arg (car pair))) (append (sxml:attr-list (car pair)) (sxml:child-nodes (car pair)))) (cdr pairs)))))))))))
(define node-parent (sxml:parent (ntype?? '*any*)))
(define sxml:child-nodes (sxml:child sxml:node?))
(define sxml:child-elements (select-kids sxml:element?))
;;; Generated from Oleg Kiselyov's "sxpath.scm"
(define (sxpath path . ns-binding) (let loop ((converters '()) (root-vars '()) (path (if (string? path) (list path) path))) (cond ((null? path) (lambda (node . root-var-binding) (let ((root-node (if (null? root-var-binding) node (car root-var-binding))) (var-binding (if (or (null? root-var-binding) (null? (cdr root-var-binding))) '() (cadr root-var-binding)))) (let rpt ((nodeset (if (nodeset? node) node (list node))) (conv (reverse converters)) (r-v (reverse root-vars))) (if (null? conv) nodeset (rpt (if (car r-v) ((car conv) nodeset root-node var-binding) ((car conv) nodeset)) (cdr conv) (cdr r-v))))))) ((and (pair? (car path)) (not (null? (car path))) (eq? 'or@ (caar path))) (loop (cons (select-kids (ntype-names?? (cdar path))) converters) (cons #f root-vars) (cdr path))) ((and (pair? (car path)) (not (null? (car path))) (eq? 'not@ (caar path))) (loop (cons (select-kids (sxml:invert (ntype-names?? (cdar path)))) converters) (cons #f root-vars) (cdr path))) ((procedure? (car path)) (loop (cons (car path) converters) (cons #t root-vars) (cdr path))) ((eq? '// (car path)) (loop (cons (node-or (node-self (ntype?? '*any*)) (node-closure (ntype?? '*any*))) converters) (cons #f root-vars) (cdr path))) ((symbol? (car path)) (loop (cons (select-kids (ntype?? (car path))) converters) (cons #f root-vars) (cdr path))) ((string? (car path)) (and-let* ((f (txpath (car path) ns-binding))) (loop (cons f converters) (cons #t root-vars) (cdr path)))) ((and (pair? (car path)) (eq? 'equal? (caar path))) (loop (cons (select-kids (apply node-equal? (cdar path))) converters) (cons #f root-vars) (cdr path))) ((and (pair? (car path)) (eq? 'ns-id:* (caar path))) (loop (cons (select-kids (ntype-namespace-id?? (cadar path))) converters) (cons #f root-vars) (cdr path))) ((and (pair? (car path)) (eq? 'eq? (caar path))) (loop (cons (select-kids (apply node-eq? (cdar path))) converters) (cons #f root-vars) (cdr path))) ((pair? (car path)) (and-let* ((select (if (symbol? (caar path)) (lambda (node root-node . var-binding) ((select-kids (ntype?? (caar path))) node)) (sxpath (caar path))))) (let reducer ((reducing-path (cdar path)) (filters '())) (cond ((null? reducing-path) (loop (cons (lambda (node root-node var-binding) (map-union (lambda (node) (let label ((nodeset (select node root-node var-binding)) (fs (reverse filters))) (if (null? fs) nodeset (label ((car fs) nodeset root-node var-binding) (cdr fs))))) (if (nodeset? node) node (list node)))) converters) (cons #t root-vars) (cdr path))) ((number? (car reducing-path)) (reducer (cdr reducing-path) (cons (lambda (node root-node var-binding) ((node-pos (car reducing-path)) node)) filters))) (else (and-let* ((func (sxpath (car reducing-path)))) (reducer (cdr reducing-path) (cons (lambda (node root-node var-binding) ((sxml:filter (lambda (n) (func n root-node var-binding))) node)) filters)))))))) (else (cerr "Invalid path step: " (car path)) #f))))
(define (if-sxpath path) (lambda (obj) (let ((x ((sxpath path) obj))) (if (null? x) #f x))))
(define (if-car-sxpath path) (lambda (obj) (let ((x ((sxpath path) obj))) (if (null? x) #f (car x)))))
(define (car-sxpath path) (lambda (obj) (let ((x ((sxpath path) obj))) (if (null? x) '() (car x)))))
(define (sxml:id-alist node . lpaths) (apply append (map (lambda (lp) (let ((lpr (reverse lp))) (map (lambda (nd) (cons (sxml:attr nd (car lpr)) nd)) ((sxpath (reverse (cons (node-self (sxpath `(|@| ,(car lpr)))) (cddr lpr)))) node)))) lpaths)))
;;; Generated from Oleg Kiselyov's "sxpath-ext.scm"
(define (sxml:string object) (cond ((string? object) object) ((nodeset? object) (if (null? object) "" (sxml:string-value (car object)))) ((number? object) (number->string object)) ((boolean? object) (if object "true" "false")) (else "")))
(define (sxml:boolean object) (cond ((boolean? object) object) ((number? object) (not (= object 0))) ((string? object) (> (string-length object) 0)) ((nodeset? object) (not (null? object))) (else #f)))
(define (sxml:number obj) (cond ((number? obj) obj) ((string? obj) (let ((nmb (call-with-input-string obj read))) (if (number? nmb) nmb 0))) ((boolean? obj) (if obj 1 0)) ((nodeset? obj) (sxml:number (sxml:string obj))) (else 0)))
(define (sxml:string-value node) (if (not (pair? node)) (if (string? node) node "") (apply string-append (cons "" (map sxml:string-value ((cond ((null? (cdr node)) cdr) ((not (pair? (cadr node))) cdr) ((not (equal? (caadr node) '|@|)) cdr) (else cddr)) node))))))
(define (sxml:id id-index) (lambda (object) (if (nodeset? object) (let loop ((str-lst (map sxml:string-value object)) (res '())) (if (null? str-lst) (reverse res) (let ((node (sxml:lookup (car str-lst) id-index))) (if (not node) (loop (cdr str-lst) res) (loop (cdr str-lst) (cons node res)))))) (let rpt ((lst (string->list (sxml:string object))) (tmp '()) (res '())) (cond ((null? lst) (if (null? tmp) (reverse res) (let ((node (sxml:lookup (list->string (reverse tmp)) id-index))) (if (not node) (reverse res) (reverse (cons node res)))))) ((member (car lst) '(#\space #\return #\newline #\tab)) (if (null? tmp) (rpt (cdr lst) tmp res) (let ((node (sxml:lookup (list->string (reverse tmp)) id-index))) (if (not node) (rpt (cdr lst) '() res) (rpt (cdr lst) '() (cons node res)))))) (else (rpt (cdr lst) (cons (car lst) tmp) res)))))))
(define (sxml:equality-cmp bool-op number-op string-op) (lambda (obj1 obj2) (cond ((and (not (nodeset? obj1)) (not (nodeset? obj2))) (cond ((boolean? obj1) (bool-op obj1 (sxml:boolean obj2))) ((boolean? obj2) (bool-op (sxml:boolean obj1) obj2)) ((number? obj1) (number-op obj1 (sxml:number obj2))) ((number? obj2) (number-op (sxml:number obj1) obj2)) (else (string-op obj1 obj2)))) ((and (nodeset? obj1) (nodeset? obj2)) (let first ((str-set1 (map sxml:string-value obj1)) (str-set2 (map sxml:string-value obj2))) (cond ((null? str-set1) #f) ((let second ((elem (car str-set1)) (set2 str-set2)) (cond ((null? set2) #f) ((string-op elem (car set2)) #t) (else (second elem (cdr set2))))) #t) (else (first (cdr str-set1) str-set2))))) (else (let-values* (((nset elem) (if (nodeset? obj1) (values obj1 obj2) (values obj2 obj1)))) (cond ((boolean? elem) (bool-op elem (sxml:boolean nset))) ((number? elem) (let loop ((nset (map (lambda (node) (sxml:number (sxml:string-value node))) nset))) (cond ((null? nset) #f) ((number-op elem (car nset)) #t) (else (loop (cdr nset)))))) ((string? elem) (let loop ((nset (map sxml:string-value nset))) (cond ((null? nset) #f) ((string-op elem (car nset)) #t) (else (loop (cdr nset)))))) (else (cerr "Unknown datatype: " elem nl) #f)))))))
(define sxml:equal? (sxml:equality-cmp eq? = string=?))
(define sxml:not-equal? (sxml:equality-cmp (lambda (bool1 bool2) (not (eq? bool1 bool2))) (lambda (num1 num2) (not (= num1 num2))) (lambda (str1 str2) (not (string=? str1 str2)))))
(define (sxml:relational-cmp op) (lambda (obj1 obj2) (cond ((not (or (nodeset? obj1) (nodeset? obj2))) (op (sxml:number obj1) (sxml:number obj2))) ((boolean? obj1) (op (sxml:number obj1) (sxml:number (sxml:boolean obj2)))) ((boolean? obj2) (op (sxml:number (sxml:boolean obj1)) (sxml:number obj2))) ((or (null? obj1) (null? obj2)) #f) (else (op (cond ((nodeset? obj1) (let ((nset1 (map (lambda (node) (sxml:number (sxml:string-value node))) obj1))) (let first ((num1 (car nset1)) (nset1 (cdr nset1))) (cond ((null? nset1) num1) ((op num1 (car nset1)) (first num1 (cdr nset1))) (else (first (car nset1) (cdr nset1))))))) ((string? obj1) (sxml:number obj1)) (else obj1)) (cond ((nodeset? obj2) (let ((nset2 (map (lambda (node) (sxml:number (sxml:string-value node))) obj2))) (let second ((num2 (car nset2)) (nset2 (cdr nset2))) (cond ((null? nset2) num2) ((op num2 (car nset2)) (second (car nset2) (cdr nset2))) (else (second num2 (cdr nset2))))))) ((string? obj2) (sxml:number obj2)) (else obj2)))))))
(define (sxml:ancestor test-pred?) (lambda (root-node) (lambda (node) (if (nodeset? node) (map-union ((sxml:ancestor test-pred?) root-node) node) (let rpt ((paths (if (nodeset? root-node) (map list root-node) (list (list root-node))))) (if (null? paths) '() (let ((path (car paths))) (if (eq? (car path) node) ((sxml:filter test-pred?) (cdr path)) (rpt (append (map (lambda (arg) (cons arg path)) (append ((sxml:attribute (ntype?? '*)) (car path)) ((sxml:child sxml:node?) (car path)))) (cdr paths)))))))))))
(define (sxml:ancestor-or-self test-pred?) (lambda (root-node) (lambda (node) (if (nodeset? node) (map-union ((sxml:ancestor-or-self test-pred?) root-node) node) (let rpt ((paths (if (nodeset? root-node) (map list root-node) (list (list root-node))))) (if (null? paths) ((sxml:filter test-pred?) (list node)) (let ((path (car paths))) (if (eq? (car path) node) ((sxml:filter test-pred?) path) (rpt (append (map (lambda (arg) (cons arg path)) (append ((sxml:attribute (ntype?? '*)) (car path)) ((sxml:child sxml:node?) (car path)))) (cdr paths)))))))))))
(define (sxml:descendant test-pred?) (lambda (node) (if (nodeset? node) (map-union (sxml:descendant test-pred?) node) (let rpt ((res '()) (more ((sxml:child sxml:node?) node))) (if (null? more) (reverse res) (rpt (if (test-pred? (car more)) (cons (car more) res) res) (append ((sxml:child sxml:node?) (car more)) (cdr more))))))))
(define (sxml:descendant-or-self test-pred?) (lambda (node) (if (nodeset? node) (map-union (sxml:descendant-or-self test-pred?) node) (let rpt ((res '()) (more (list node))) (if (null? more) (reverse res) (rpt (if (test-pred? (car more)) (cons (car more) res) res) (append ((sxml:child sxml:node?) (car more)) (cdr more))))))))
(define (sxml:following test-pred?) (lambda (root-node) (lambda (node) (if (nodeset? node) (map-union ((sxml:following test-pred?) root-node) node) (let loop ((seq (if (nodeset? root-node) (list root-node) (list (list root-node))))) (cond ((null? seq) '()) ((null? (car seq)) (loop (cdr seq))) ((eq? (caar seq) node) (let rpt ((seq (cdr (apply append seq))) (res '())) (if (null? seq) res (rpt (cdr seq) (append res ((sxml:descendant-or-self test-pred?) (car seq))))))) ((and (sxml:element? (caar seq)) (memq node (sxml:attr-list (caar seq)))) (let rpt ((sq (cdr (apply append seq))) (res ((sxml:descendant test-pred?) (caar seq)))) (if (null? sq) res (rpt (cdr sq) (append res ((sxml:descendant-or-self test-pred?) (car sq))))))) (else (loop (cons ((sxml:child sxml:node?) (caar seq)) (cons (cdar seq) (cdr seq)))))))))))
(define (sxml:following-sibling test-pred?) (lambda (root-node) (lambda (node) (if (nodeset? node) (map-union ((sxml:following-sibling test-pred?) root-node) node) (let loop ((seqs (if (nodeset? root-node) (list root-node) (list (list root-node))))) (if (null? seqs) '() (let rpt ((seq (car seqs))) (cond ((null? seq) (loop (append (map (sxml:child sxml:node?) (car seqs)) (cdr seqs)))) ((eq? (car seq) node) ((sxml:filter test-pred?) (cdr seq))) (else (rpt (cdr seq)))))))))))
(define (sxml:namespace test-pred?) (lambda (node) ((sxml:filter test-pred?) (sxml:ns-list node))))
(define (sxml:preceding test-pred?) (lambda (root-node) (lambda (node) (if (nodeset? node) (map-union ((sxml:preceding test-pred?) root-node) node) (let loop ((seq (if (nodeset? root-node) (list (reverse root-node)) (list (list root-node))))) (cond ((null? seq) '()) ((null? (car seq)) (loop (cdr seq))) ((or (eq? (caar seq) node) (not (null? ((sxml:attribute (lambda (n) (eq? n node))) (caar seq))))) (let rpt ((seq (cdr (apply append seq))) (res '())) (if (null? seq) res (rpt (cdr seq) (append res (reverse ((sxml:descendant-or-self test-pred?) (car seq)))))))) (else (loop (cons (reverse ((sxml:child sxml:node?) (caar seq))) (cons (cdar seq) (cdr seq)))))))))))
(define (sxml:preceding-sibling test-pred?) (lambda (root-node) (lambda (node) (if (nodeset? node) (map-union ((sxml:preceding-sibling test-pred?) root-node) node) (let loop ((seqs (if (nodeset? root-node) (list root-node) (list (list root-node))))) (if (null? seqs) '() (let rpt ((seq (car seqs))) (cond ((null? seq) (loop (append (map (lambda (n) (reverse ((sxml:child sxml:node?) n))) (car seqs)) (cdr seqs)))) ((eq? (car seq) node) ((sxml:filter test-pred?) (cdr seq))) (else (rpt (cdr seq)))))))))))

(provide "sxml/sxpath")

          
;; Local variables:
;; mode: scheme
;; end:
